package com.ruoyi.common.utils;

import java.lang.management.ManagementFactory;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.util.Calendar;
import java.util.Date;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang3.time.DateFormatUtils;

/**
 * 时间工具类
 *
 * @author ruoyi
 */
public class DateUtils extends org.apache.commons.lang3.time.DateUtils
{
    public static String YYYY = "yyyy";

    public static String YYYY_MM = "yyyy-MM";

    public static String YYYY_MM_DD = "yyyy-MM-dd";

    public static String YYYYMMDDHHMMSS = "yyyyMMddHHmmss";

    public static String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";

    public static final String DATE_FORMAT_HHMMSS = "HH:mm:ss";

    private static String[] parsePatterns = {
            "yyyy-MM-dd", "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd HH:mm", "yyyy-MM",
            "yyyy/MM/dd", "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd HH:mm", "yyyy/MM",
            "yyyy.MM.dd", "yyyy.MM.dd HH:mm:ss", "yyyy.MM.dd HH:mm", "yyyy.MM"};

    // 定义日期格式
    private static final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

    /**
     * 获取当前Date型日期
     *
     * @return Date() 当前日期
     */
    public static Date getNowDate()
    {
        return new Date();
    }

    /**
     * 获取昨天的Date型日期
     *
     * @return Date() 昨天的日期
     */
    public static Date getYesterdayDate() {
        return Date.from(LocalDate.now().minusDays(1).atStartOfDay(ZoneId.systemDefault()).toInstant());
    }

    /**
     * 获取昨天的Date型日期
     *
     * @return Date() 昨天的日期
     */
    public static LocalDate getYesterdayLocalDate() {
        return LocalDate.now().minusDays(1);
    }

    /**
     * 获取当前日期, 默认格式为yyyy-MM-dd
     *
     * @return String
     */
    public static String getDate()
    {
        return dateTimeNow(YYYY_MM_DD);
    }

    public static final String getTime()
    {
        return dateTimeNow(YYYY_MM_DD_HH_MM_SS);
    }

    public static final String dateTimeNow()
    {
        return dateTimeNow(YYYYMMDDHHMMSS);
    }

    public static final String dateTimeNow(final String format)
    {
        return parseDateToStr(format, new Date());
    }

    public static final String dateTime(final Date date)
    {
        return parseDateToStr(YYYY_MM_DD, date);
    }

    public static final String parseDateToStr(final String format, final Date date)
    {
        return new SimpleDateFormat(format).format(date);
    }

    public static final Date dateTime(final String format, final String ts)
    {
        try
        {
            return new SimpleDateFormat(format).parse(ts);
        }
        catch (ParseException e)
        {
            throw new RuntimeException(e);
        }
    }

    /**
     * 日期路径 即年/月/日 如2018/08/08
     */
    public static final String datePath()
    {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyy/MM/dd");
    }

    /**
     * 日期路径 即年/月/日 如20180808
     */
    public static final String dateTime()
    {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyyMMdd");
    }

    /**
     * 日期路径 即年/月/日 如20180808
     */
    public static final String getYYYY_MM_DD(Date date)
    {
        return DateFormatUtils.format(date, YYYY_MM_DD);
    }

    public static String getYYYY_MM_DD(LocalDate localDate) {
        return format(localDate, YYYY_MM_DD);
    }

    private static String format(LocalDate localDate, String format) {
        if (localDate == null) {
            return null;
        }
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(format);
        return localDate.format(formatter);
    }

    /**
     * 日期型字符串转化为日期 格式
     */
    public static Date parseDate(Object str)
    {
        if (str == null)
        {
            return null;
        }
        try
        {
            return parseDate(str.toString(), parsePatterns);
        }
        catch (ParseException e)
        {
            return null;
        }
    }

    /**
     * 获取服务器启动时间
     */
    public static Date getServerStartDate()
    {
        long time = ManagementFactory.getRuntimeMXBean().getStartTime();
        return new Date(time);
    }

    /**
     * 计算相差天数
     */
    public static int differentDaysByMillisecond(Date date1, Date date2)
    {
        return Math.abs((int) ((date2.getTime() - date1.getTime()) / (1000 * 3600 * 24)));
    }

    /**
     * 计算相差天数
     */
    public static int differentDaysByMillisecond(String date1Str, String date2Str)
    {
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        try {
            Date date1 = formatter.parse(date1Str);
            Date date2 = formatter.parse(date2Str);
            return Math.abs((int) ((date2.getTime() - date1.getTime()) / (1000 * 3600 * 24)));
        } catch (ParseException e) {
            throw new IllegalArgumentException("无效的日期格式。格式为yyyy-MM-dd", e);
        }
    }

    /**
     * 计算时间差
     *
     * @param endDate 最后时间
     * @param startTime 开始时间
     * @return 时间差（天/小时/分钟）
     */
    public static String timeDistance(Date endDate, Date startTime)
    {
        long nd = 1000 * 24 * 60 * 60;
        long nh = 1000 * 60 * 60;
        long nm = 1000 * 60;
        // long ns = 1000;
        // 获得两个时间的毫秒时间差异
        long diff = endDate.getTime() - startTime.getTime();
        // 计算差多少天
        long day = diff / nd;
        // 计算差多少小时
        long hour = diff % nd / nh;
        // 计算差多少分钟
        long min = diff % nd % nh / nm;
        // 计算差多少秒//输出结果
        // long sec = diff % nd % nh % nm / ns;
        return day + "天" + hour + "小时" + min + "分钟";
    }

    /**
     * 计算时间差（只返回小时数）
     *
     * @param endDate 结束时间
     * @param startTime 开始时间
     * @return 时间差（小时）
     */
    public static int timeDifferenceInHours(Date endDate, Date startTime) {
        // 获得两个时间的毫秒时间差异
        long diff = endDate.getTime() - startTime.getTime();
        // 直接计算差多少小时
        return (int) Math.abs(diff / (1000 * 60 * 60));
    }

    /**
     * 计算时间差（向上取整到小时数）
     *
     * @param endDate 结束时间
     * @param startTime 开始时间
     * @return 时间差（小时）
     */
    public static int timeDifferenceInHoursWithCeil(Date endDate, Date startTime) {
        // 获得两个时间的毫秒时间差异
        long diff = endDate.getTime() - startTime.getTime();
        // 将毫秒差异转换为小时的小数形式
        double hoursDiff = diff / (1000.0 * 60 * 60);
        // 使用 Math.ceil() 向上取整到最近的整数小时
        return (int) Math.ceil(Math.abs(hoursDiff));
    }

    /**
     * 计算时间差（只返回分钟数）
     *
     * @param endDate 结束时间
     * @param startTime 开始时间
     * @return 时间差（分钟）
     */
    public static int timeDifferenceInMinutes(Date endDate, Date startTime) {
        // 获得两个时间的毫秒时间差异
        long diff = endDate.getTime() - startTime.getTime();

        // 直接计算差多少分钟
        return (int) Math.abs(diff / (1000 * 60));
    }

    /**
     * 计算时间差（只返回秒数）
     *
     * @param endDate 结束时间
     * @param startTime 开始时间
     * @return 时间差（秒）
     */
    public static long timeDifferenceInSeconds(Date endDate, Date startTime) {
        // 获得两个时间的毫秒时间差异
        long diff = endDate.getTime() - startTime.getTime();

        // 直接计算差多少秒
        return Math.abs(diff / 1000);
    }

    /**
     * 增加 LocalDateTime ==> Date
     */
    public static Date toDate(LocalDateTime temporalAccessor)
    {
        ZonedDateTime zdt = temporalAccessor.atZone(ZoneId.systemDefault());
        return Date.from(zdt.toInstant());
    }

    /**
     * 增加 LocalDate ==> Date
     */
    public static Date toDate(LocalDate temporalAccessor)
    {
        LocalDateTime localDateTime = LocalDateTime.of(temporalAccessor, LocalTime.of(0, 0, 0));
        ZonedDateTime zdt = localDateTime.atZone(ZoneId.systemDefault());
        return Date.from(zdt.toInstant());
    }

    /**
     * 减去多少天后的时间
     */
    public static Date getTimeAfterXDays(Date unlockTime, int xDays) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(unlockTime);
        calendar.add(Calendar.DAY_OF_MONTH, -xDays);
        return calendar.getTime(); // 返回新时间
    }

    /**
     * 加上固定个小时后的时间
     */
    public static Date getTimeAfterXHours(Date unlockTime, int xHours) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(unlockTime);
        calendar.add(Calendar.HOUR_OF_DAY, xHours); // 加上X小时
        return calendar.getTime(); // 返回新时间
    }

    /**
     * 加上固定个分钟后的时间
     */
    public static Date getTimeAfterXMinutes(Date originalTime, int xMinutes) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(originalTime);
        calendar.add(Calendar.MINUTE, xMinutes); // 加上X分钟
        return calendar.getTime(); // 返回新时间
    }

    /**
     * 判断是否在时间范围内
     */
    public static Boolean isInTime(String areaTimeStart, String areaTimeEnd) {
        String currentTime = new SimpleDateFormat(DATE_FORMAT_HHMMSS).format(new Date());
        if (areaTimeStart.compareTo(currentTime) <= 0 && areaTimeEnd.compareTo(currentTime) >= 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 判断是否在时间范围内
     */
    public static Boolean isInTime(Date areaTimeStart, Date areaTimeEnd) {
        if (areaTimeStart.compareTo(new Date()) <= 0 && areaTimeEnd.compareTo(new Date()) >= 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 判断合作时间是否过期
     */
    public static boolean isCooperationExpired(Date cooperationTime) {
        Date currentTime = new Date();
        return currentTime.after(cooperationTime);
    }

    /**
     * 根据num和rentalUnit计算新的时间
     *
     * @param startTime  开始时间（例如取车时间）
     * @param num        时间单位数量
     * @param rentalUnit 租赁单位（hours, day, week, month, quarter, year）
     * @return           计算后的新时间
     */
    public static Date getTimeAfterXUnit(Date startTime, int num, String rentalUnit) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startTime);

        switch (rentalUnit) {
            case "hours":
                calendar.add(Calendar.HOUR_OF_DAY, num);
                break;
            case "day":
                calendar.add(Calendar.DAY_OF_YEAR, num);
                break;
            case "week":
                calendar.add(Calendar.WEEK_OF_YEAR, num);
                break;
            case "month":
                calendar.add(Calendar.MONTH, num);
                break;
            case "quarter":
                calendar.add(Calendar.MONTH, num * 3); // 一个季度等于三个月
                break;
            case "year":
                calendar.add(Calendar.YEAR, num);
                break;
            default:
                throw new IllegalArgumentException("无效的rentalUnit值：" + rentalUnit);
        }

        return calendar.getTime(); // 返回新的时间
    }

    /**
     * ISO 字符串转换为LocalDateTime
     * @param time 时间字符串:yyyy-MM-dd HH:mm:ss
     */
    public static LocalDateTime toLocalDateTimeByISO(String time) {
        DateTimeFormatter formatter = DateTimeFormatter.ISO_ZONED_DATE_TIME;
        return LocalDateTime.parse(time, formatter);
    }

    public static LocalDateTime toLocalDateTime(Date date) {
        return date.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();
    }

    public static LocalTime toLocalTime(Date date) {
        return date.toInstant().atZone(ZoneId.systemDefault()).toLocalTime();
    }

    /**
     * RFC3339格式转为Date
     */
    public static Date toDateByRfc3339(String str) {
        DateTimeFormatter formatter = DateTimeFormatter.ISO_OFFSET_DATE_TIME;
        OffsetDateTime dateTime = OffsetDateTime.parse(str, formatter);
        return Date.from(dateTime.toInstant());
    }


    public static LocalDate stringToLocalDate(String day) {
        return LocalDate.parse(day, formatter);
    }

    /**
     *  解析带有时区偏移量的日期时间字符串
     */
    public static Date parseTime(String timeStr) {
        OffsetDateTime offsetDateTime = OffsetDateTime.parse(timeStr);
        ZonedDateTime zonedDateTime = offsetDateTime.toZonedDateTime();
        return Date.from(zonedDateTime.toInstant());
    }
}
