package com.ruoyi.ss.dividendDetail.mapper;

import com.ruoyi.ss.device.domain.DeviceQuery;
import com.ruoyi.ss.dividendDetail.domain.DividendDetail;
import com.ruoyi.ss.dividendDetail.domain.DividendDetailQuery;
import com.ruoyi.ss.dividendDetail.domain.DividendDetailVO;
import com.ruoyi.system.domain.vo.InCome;
import com.ruoyi.system.domain.vo.IndexAdminVo;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;


/**
 * 分账明细Mapper接口
 *
 * @author qzz
 * @date 2024-10-10
 */
public interface DividendDetailMapper
{
    /**
     * 查询分账明细
     *
     * @param id 分账明细主键
     * @return 分账明细
     */
    public DividendDetail selectRlDividendDetailById(Long id);

    /**
     * 查询分账明细列表
     *
     * @param rlDividendDetail 分账明细
     * @return 分账明细集合
     */
    public List<DividendDetailVO> selectRlDividendDetailList(DividendDetailQuery rlDividendDetail);

    /**
     * 新增分账明细
     *
     * @param rlDividendDetail 分账明细
     * @return 结果
     */
    public int insertRlDividendDetail(DividendDetail rlDividendDetail);

    /**
     * 修改分账明细
     *
     * @param rlDividendDetail 分账明细
     * @return 结果
     */
    public int updateRlDividendDetail(DividendDetail rlDividendDetail);

    /**
     * 删除分账明细
     *
     * @param id 分账明细主键
     * @return 结果
     */
    public int deleteRlDividendDetailById(Long id);

    /**
     * 批量删除分账明细
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteRlDividendDetailByIds(Long[] ids);

    /**
     * 根据订单号查询分账明细列表
     * @param orderNo
     * @return
     */
    List<DividendDetailVO> selectRlDividendDetailListByOrderNo(String orderNo);

    /**
     * 根据商户id查询近7天收入
     */
    List<InCome> select7DaysIncome(Long merchantId);

    /**
     * 根据商户id查询近30天收入
     */
    List<InCome> select30DaysIncome(Long merchantId);

    /**
     * 查询收入
     */
    BigDecimal getIncome(DividendDetailQuery query );

    /**
     * 根据时间查询收入
     */
    List<InCome> selectIncome(DividendDetailQuery query);
}
