package com.ruoyi.ss.dividendDetail.service;

import com.ruoyi.ss.dividendDetail.domain.DividendDetail;
import com.ruoyi.ss.dividendDetail.domain.DividendDetailQuery;
import com.ruoyi.ss.dividendDetail.domain.DividendDetailVO;
import com.ruoyi.system.domain.vo.InCome;
import com.ruoyi.system.domain.vo.IndexAdminVo;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;

/**
 * 分账明细Service接口
 *
 * @author qzz
 * @date 2024-10-10
 */
public interface IDividendDetailService
{
    /**
     * 查询分账明细
     *
     * @param id 分账明细主键
     * @return 分账明细
     */
    public DividendDetail selectRlDividendDetailById(Long id);

    /**
     * 查询分账明细列表
     *
     * @param rlDividendDetail 分账明细
     * @return 分账明细集合
     */
    public List<DividendDetailVO> selectRlDividendDetailList(DividendDetailQuery rlDividendDetail);

    /**
     * 根据订单号查询分账明细列表
     * @param orderNo 订单号
     * @return 分账明细集合
     */
    public List<DividendDetailVO> selectRlDividendDetailListByOrderNo(String orderNo);

    /**
     * 新增分账明细
     *
     * @param rlDividendDetail 分账明细
     * @return 结果
     */
    public int insertRlDividendDetail(DividendDetail rlDividendDetail);

    /**
     * 修改分账明细
     *
     * @param rlDividendDetail 分账明细
     * @return 结果
     */
    public int updateRlDividendDetail(DividendDetail rlDividendDetail);

    /**
     * 批量删除分账明细
     *
     * @param ids 需要删除的分账明细主键集合
     * @return 结果
     */
    public int deleteRlDividendDetailByIds(Long[] ids);

    /**
     * 删除分账明细信息
     *
     * @param id 分账明细主键
     * @return 结果
     */
    public int deleteRlDividendDetailById(Long id);

//    /**
//     * 记录分成比例
//     *
//     * @param originalOrder 原始订单对象
//     * @param userType 用户类型：商户，代理商
//     * @return 结果
//     */
//    public int calculationDividend(RlOrder originalOrder,String userType);
//
//    /**
//     * 计算分成金额
//     *
//     * @param rlDividendDetails 分成明细列表
//     * @param totalDividendAmount 分账总金额
//     * @return 结果
//     */
//    public int calculationAmount(List<RlDividendDetailVO> rlDividendDetails, BigDecimal totalDividendAmount);

    /**
     * 更新退款金额
     *
     * @param id 分成明细id
     * @param dividendAmount 分账金额（为了计算结算金额）
     * @param userRefundAmount 用户退款金额
     * @return 结果
     */
    public int updateRefundAmount(Long id, BigDecimal dividendAmount, BigDecimal userRefundAmount);

    /**
     * 查询近7天收入
     * @param merchantId 商户id
     * @return 结果
     */
    List<InCome> select7DaysIncome(Long merchantId);

    /**
     * 查询近30天收入
     * @param merchantId 商户id
     * @return 结果
     */
    List<InCome> select30DaysIncome(Long merchantId);

    /**
     * 根据时间查询收入
     * @param query 查询条件
     * @return 结果
     */
    List<InCome> selectIncome(DividendDetailQuery query);


    /**
     * 获取指定时间范围的收入
     * @return 收入金额
     */
    BigDecimal getIncome(DividendDetailQuery query);
}
