package com.ruoyi.ss.mapping.service.impl;

import java.util.List;

import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.ss.mapping.mapper.StoreMappingMapper;
import com.ruoyi.ss.mapping.domain.StoreMapping;
import com.ruoyi.ss.mapping.domain.StoreMappingVO;
import com.ruoyi.ss.mapping.domain.StoreMappingQuery;
import com.ruoyi.ss.mapping.service.IStoreMappingService;

/**
 * 系统与第三方店铺映射关系Service业务层处理
 *
 * @author qzz
 * @date 2025-03-18
 */
@Service
public class StoreMappingServiceImpl implements IStoreMappingService
{
    @Autowired
    private StoreMappingMapper storeMappingMapper;

    /**
     * 查询系统与第三方店铺映射关系
     *
     * @param storeId 系统与第三方店铺映射关系主键
     * @return 系统与第三方店铺映射关系
     */
    @Override
    public StoreMappingVO selectStoreMappingByStoreId(Long storeId)
    {
        return storeMappingMapper.selectStoreMappingByStoreId(storeId);
    }

    /**
     * 查询系统与第三方店铺映射关系列表
     *
     * @param storeMapping 系统与第三方店铺映射关系
     * @return 系统与第三方店铺映射关系
     */
    @Override
    public List<StoreMappingVO> selectStoreMappingList(StoreMappingQuery storeMapping)
    {
        return storeMappingMapper.selectStoreMappingList(storeMapping);
    }

    /**
     * 新增系统与第三方店铺映射关系
     *
     * @param storeMapping 系统与第三方店铺映射关系
     * @return 结果
     */
    @Override
    public int insertStoreMapping(StoreMapping storeMapping)
    {
        storeMapping.setCreateTime(DateUtils.getNowDate());
        return storeMappingMapper.insertStoreMapping(storeMapping);
    }

    /**
     * 修改系统与第三方店铺映射关系
     *
     * @param storeMapping 系统与第三方店铺映射关系
     * @return 结果
     */
    @Override
    public int updateStoreMapping(StoreMapping storeMapping)
    {
        return storeMappingMapper.updateStoreMapping(storeMapping);
    }

    /**
     * 批量删除系统与第三方店铺映射关系
     *
     * @param storeIds 需要删除的系统与第三方店铺映射关系主键
     * @return 结果
     */
    @Override
    public int deleteStoreMappingByStoreIds(Long[] storeIds)
    {
        return storeMappingMapper.deleteStoreMappingByStoreIds(storeIds);
    }

    /**
     * 删除系统与第三方店铺映射关系信息
     *
     * @param storeId 系统与第三方店铺映射关系主键
     * @return 结果
     */
    @Override
    public int deleteStoreMappingByStoreId(Long storeId)
    {
        return storeMappingMapper.deleteStoreMappingByStoreId(storeId);
    }
}
