package com.ruoyi.ss.menu.service;

import com.ruoyi.ss.menu.domain.Menu;
import com.ruoyi.ss.menu.domain.MenuQuery;
import com.ruoyi.ss.menu.domain.MenuVO;

import java.util.List;

/**
 * 前端菜单Service接口
 *
 * @author qzz
 * @date 2025-01-16
 */
public interface IMenuService
{
    /**
     * 查询前端菜单
     *
     * @param menuId 前端菜单主键
     * @return 前端菜单
     */
    public MenuVO selectSsMenuByMenuId(Long menuId);

    /**
     * 查询前端菜单列表
     *
     * @param ssMenu 前端菜单
     * @return 前端菜单集合
     */
    public List<MenuVO> selectSsMenuList(MenuQuery ssMenu);

    /**
     * 根据用户角色查询所有菜单
     *
     * @param role 用户类型
     * @return 前端菜单集合
     */
    public List<MenuVO> selectEtMenuListByRole(String role);


    /**
     * 新增前端菜单
     *
     * @param menu 前端菜单
     * @return 结果
     */
    public int insertSsMenu(Menu menu);

    /**
     * 修改前端菜单
     *
     * @param ssMenu 前端菜单
     * @return 结果
     */
    public int updateSsMenu(Menu ssMenu);

    /**
     * 批量删除前端菜单
     *
     * @param menuIds 需要删除的前端菜单主键集合
     * @return 结果
     */
    public int deleteSsMenuByMenuIds(Long[] menuIds);

    /**
     * 删除前端菜单信息
     *
     * @param menuId 前端菜单主键
     * @return 结果
     */
    public int deleteSsMenuByMenuId(Long menuId);
}
