package com.ruoyi.ss.menu.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.ss.menu.domain.Menu;
import com.ruoyi.ss.menu.domain.MenuQuery;
import com.ruoyi.ss.menu.domain.MenuVO;
import com.ruoyi.ss.menu.mapper.MenuMapper;
import com.ruoyi.ss.menu.service.IMenuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 前端菜单Service业务层处理
 *
 * @author qzz
 * @date 2025-01-16
 */
@Service
public class MenuServiceImpl implements IMenuService
{
    @Autowired
    private MenuMapper menuMapper;

    /**
     * 查询前端菜单
     *
     * @param menuId 前端菜单主键
     * @return 前端菜单
     */
    @Override
    public MenuVO selectSsMenuByMenuId(Long menuId)
    {
        return menuMapper.selectSsMenuByMenuId(menuId);
    }

    /**
     * 查询前端菜单列表
     *
     * @param menu 前端菜单
     * @return 前端菜单
     */
    @Override
    public List<MenuVO> selectSsMenuList(MenuQuery menu)
    {
        menu.setStatus("0");
        return menuMapper.selectSsMenuList(menu);
    }

    /**
     * 根据用户角色查询所有菜单
     *
     * @param role 角色
     * @return 前端菜单
     */
    @Override
    public List<MenuVO> selectEtMenuListByRole(String role) {
        return menuMapper.selectEtMenuListByRole(role);
    }

    /**
     * 新增前端菜单
     *
     * @param menu 前端菜单
     * @return 结果
     */
    @Override
    public int insertSsMenu(Menu menu)
    {
        menu.setCreateTime(DateUtils.getNowDate());
        return menuMapper.insertSsMenu(menu);
    }

    /**
     * 修改前端菜单
     *
     * @param menu 前端菜单
     * @return 结果
     */
    @Override
    public int updateSsMenu(Menu menu)
    {
        return menuMapper.updateSsMenu(menu);
    }

    /**
     * 批量删除前端菜单
     *
     * @param menuIds 需要删除的前端菜单主键
     * @return 结果
     */
    @Override
    public int deleteSsMenuByMenuIds(Long[] menuIds)
    {
        return menuMapper.deleteSsMenuByMenuIds(menuIds);
    }

    /**
     * 删除前端菜单信息
     *
     * @param menuId 前端菜单主键
     * @return 结果
     */
    @Override
    public int deleteSsMenuByMenuId(Long menuId)
    {
        return menuMapper.deleteSsMenuByMenuId(menuId);
    }
}
