package com.ruoyi.ss.onlineLog.service.impl;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.ss.onlineLog.domain.OnlineLog;
import com.ruoyi.ss.onlineLog.mapper.OnlineLogMapper;
import com.ruoyi.ss.onlineLog.service.IOnlineLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 在线/离线日志Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-07-13
 */
@Service
public class OnlineLogServiceImpl implements IOnlineLogService
{
    @Autowired
    private OnlineLogMapper eOnlineLogMapper;

    /**
     * 查询在线/离线日志
     *
     * @param id 在线/离线日志主键
     * @return 在线/离线日志
     */
    @Override
    public OnlineLog selectEtOnlineLogById(Long id)
    {
        return eOnlineLogMapper.selectEtOnlineLogById(id);
    }

    /**
     * 查询在线/离线日志列表
     *
     * @param eOnlineLog 在线/离线日志
     * @return 在线/离线日志
     */
    @Override
    public List<OnlineLog> selectEtOnlineLogList(OnlineLog eOnlineLog)
    {
        return eOnlineLogMapper.selectEtOnlineLogList(eOnlineLog);
    }

    /**
     * 新增在线/离线日志
     *
     * @param eOnlineLog 在线/离线日志
     * @return 结果
     */
    @Override
    public int insertEtOnlineLog(OnlineLog eOnlineLog)
    {
        eOnlineLog.setCreateTime(DateUtils.getNowDate());
        return eOnlineLogMapper.insertEtOnlineLog(eOnlineLog);
    }

    /**
     * 修改在线/离线日志
     *
     * @param eOnlineLog 在线/离线日志
     * @return 结果
     */
    @Override
    public int updateEtOnlineLog(OnlineLog eOnlineLog)
    {
        return eOnlineLogMapper.updateEtOnlineLog(eOnlineLog);
    }

    /**
     * 批量删除在线/离线日志
     *
     * @param ids 需要删除的在线/离线日志主键
     * @return 结果
     */
    @Override
    public int deleteEtOnlineLogByIds(Long[] ids)
    {
        return eOnlineLogMapper.deleteEtOnlineLogByIds(ids);
    }

    /**
     * 删除在线/离线日志信息
     *
     * @param id 在线/离线日志主键
     * @return 结果
     */
    @Override
    public int deleteEtOnlineLogById(Long id)
    {
        return eOnlineLogMapper.deleteEtOnlineLogById(id);
    }
}
