package com.ruoyi.ss.product.mapper;

import java.util.List;

import com.ruoyi.ss.product.domain.Product;
import com.ruoyi.ss.product.domain.ProductQuery;
import com.ruoyi.ss.product.domain.ProductVO;
import org.apache.ibatis.annotations.Param;

/**
 * 商品列表Mapper接口
 *
 * @author qzz
 * @date 2024-12-16
 */
public interface ProductMapper
{
    /**
     * 查询商品列表
     *
     * @param productId 商品列表主键
     * @return 商品列表
     */
    public ProductVO selectProductByProductId(Long productId);

    /**
     * 查询商品列表列表
     *
     * @param query 商品列表
     * @return 商品列表集合
     */
    public List<ProductVO> selectProductList(@Param("query") ProductQuery query);

    /**
     * 新增商品列表
     *
     * @param product 商品列表
     * @return 结果
     */
    public int insertProduct(Product product);

    /**
     * 修改商品列表
     *
     * @param product 商品列表
     * @return 结果
     */
    public int updateProduct(@Param("data") Product product);

    /**
     * 删除商品列表
     *
     * @param productId 商品列表主键
     * @return 结果
     */
    public int deleteProductByProductId(Long productId);

    /**
     * 批量删除商品列表
     *
     * @param productIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteProductByProductIds(Long[] productIds);

    /**
     * 更新库存减一
     *
     * @param productId 商品列表主键
     * @return 结果
     */
    int updateProductStock(Long productId);
}
