package com.ruoyi.ss.product.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.ss.product.domain.Product;
import com.ruoyi.ss.product.domain.ProductVO;
import com.ruoyi.ss.product.domain.ProductQuery;
import com.ruoyi.ss.product.mapper.ProductMapper;
import com.ruoyi.ss.product.service.IProductService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import static com.ruoyi.common.utils.SecurityUtils.getUsername;

/**
 * 商品列表Service业务层处理
 *
 * @author qzz
 * @date 2024-12-16
 */
@Service
public class ProductServiceImpl implements IProductService
{
    @Autowired
    private ProductMapper productMapper;

    /**
     * 查询商品列表
     *
     * @param productId 商品列表主键
     * @return 商品列表
     */
    @Override
    public ProductVO selectProductByProductId(Long productId)
    {
        return productMapper.selectProductByProductId(productId);
    }

    /**
     * 查询商品列表列表
     *
     * @param product 商品列表
     * @return 商品列表
     */
    @Override
    public List<ProductVO> selectProductList(ProductQuery product)
    {
        return productMapper.selectProductList(product);
    }

    /**
     * 新增商品列表
     *
     * @param product 商品列表
     * @return 结果
     */
    @Override
    public int insertProduct(Product product)
    {
        product.setCreateTime(DateUtils.getNowDate());
        product.setCreateBy(getUsername());
        return productMapper.insertProduct(product);
    }

    /**
     * 修改商品列表
     *
     * @param product 商品列表
     * @return 结果
     */
    @Override
    public int updateProduct(Product product)
    {
        product.setUpdateTime(DateUtils.getNowDate());
        return productMapper.updateProduct(product);
    }

    /**
     * 批量删除商品列表
     *
     * @param productIds 需要删除的商品列表主键
     * @return 结果
     */
    @Override
    public int deleteProductByProductIds(Long[] productIds)
    {
        return productMapper.deleteProductByProductIds(productIds);
    }

    /**
     * 删除商品列表信息
     *
     * @param productId 商品列表主键
     * @return 结果
     */
    @Override
    public int deleteProductByProductId(Long productId)
    {
        return productMapper.deleteProductByProductId(productId);
    }

    /**
     * 更新库存减一
     *
     * @param productId 商品列表主键
     * @return 结果
     */
    @Override
    public int updateProductStock(Long productId) {
        return productMapper.updateProductStock(productId);
    }
}
