package com.ruoyi.system.iot.service;


import com.ruoyi.ss.device.domain.enums.DeviceOnlineStatus;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.system.iot.domain.HistoryDeviceData;
import com.ruoyi.system.iot.domain.IotDeviceInfo;
import com.ruoyi.system.iot.domain.response.CommandResponse;
import com.ruoyi.system.iot.interfaces.IotDevice;

import java.math.BigDecimal;
import java.util.List;

/**
 * @author qzz
 * 2024/3/20
 */
public interface IotService {

    /**
     * 获取设备在线状态
     *
     * @param deviceName OneNet设备名称（即设备表的MAC号）
     * @param productId
     */
    String getOnlineStatus(String deviceName, String productId);

    /**
     * 获取设备在线状态
     */
    String getOnlineStatus(IotDevice device);

    /**
     * 通电
     *
     * @param deviceName OneNet设备名称（即设备表的MAC号）
     * @param productId
     * @return 是否成功
     */
    CommandResponse open(String deviceName, String productId,String reason);

    /**
     * 断电
     *
     * @param deviceName OneNet设备名称（即设备表的MAC号）
     * @param productId
     * @return 是否成功
     */
    CommandResponse close(String deviceName, String productId,String reason);

    /**
     * 通电
     */
    boolean open(IotDevice device,boolean isIgnore,String reason,boolean returnMac);

    /**
     * 重启设备
     */
    CommandResponse reboot(IotDevice device, String reason);

    /**
     * 开门
     */
    CommandResponse openDoor(IotDevice device, String reason, boolean open);

    /**
     * 断电
     *
     * @param deviceName OneNet设备名称（即设备表的MAC号）
     * @param productId
     * @return 是否成功
     */
    boolean close(String deviceName, String productId,boolean isIgnore,String reason);

    /**
     * 断电
     */
    boolean close(IotDevice device, boolean isIgnore, String reason,boolean returnMac);

    /**
     * 设置通电时长
     *
     * @param deviceName 设备名称
     * @param seconds    时长（秒）
     * @param productId
     */
    CommandResponse setTime(String deviceName, long seconds, String productId,String reason);

    /**
     * 设置断电时长
     *
     * @param deviceName 设备名称
     * @param seconds    时长（秒）
     * @param productId
     */
    CommandResponse setTtme(String deviceName, long seconds, String productId, String reason);

    /**
     * 设置通电时长
     */
    CommandResponse setTime(IotDevice device, long seconds, String reason);

    /**
     * 开门
     */
    CommandResponse km(IotDevice device, long seconds, String reason, boolean returnMac);

    /**
     * 关门
     */
    CommandResponse gm(IotDevice device, long seconds, String reason, boolean returnMac);

    /**
     * 设置断电时长
     */
    CommandResponse setTtme(IotDevice device, long seconds, String reason);

    /**
     * 获取设备信息，并转化为IotDeviceInfo
     *
     * @param deviceName 设备名称
     * @param productId
     */
    IotDeviceInfo getDeviceInfo(String deviceName, String productId);

    /**
     * 获取设备信息，并转为IotDeviceInfo
     */
    IotDeviceInfo getDeviceInfo(IotDevice device);

    /**
     * 获取设备信息列表，并转换为IotDeviceInfo
     *
     * @param deviceList 设备名称列表
     * @param productId
     */
    List<IotDeviceInfo> getDeviceInfo(List<? extends IotDevice> deviceList, String productId);

    /**
     * 注册设备
     */
    int create(String mac, String productId);

    /**
     * 设备添加电量（度）
     */
    CommandResponse addEle(String deviceName, BigDecimal ele, String productId);

    /**
     * 设备添加电量（度）
     */
    CommandResponse addEle(IotDevice device, BigDecimal ele);

    /**
     * 直接设置设备电量（度）
     */
    CommandResponse setEle(String deviceName, BigDecimal ele, String productId);

    /**
     * 直接设置设备电量（度）
     */
    CommandResponse setEle(IotDevice device, BigDecimal ele);

    /**
     * 尝试设置设备剩余时长
     *
     * @param device   MAC
     * @param seconds  时长
     * @param tryCount 尝试次数
     */
    CommandResponse trySetTime(IotDevice device, long seconds, int tryCount, String reason);

    /**
     * 尝试设置设备剩余时长
     *
     * @param device   MAC
     * @param ele      电量（度）
     * @param tryCount 尝试次数
     */
    CommandResponse trySetEle(IotDevice device, BigDecimal ele, int tryCount);

    /**
     * 设置设备WIFI
     * @param device 设备
     * @param wifiName WIFI名称
     * @param wifiPwd WIFI 密码
     */
    CommandResponse setWifi(IotDevice device, String wifiName, String wifiPwd);

    CommandResponse setWifi(String mac, String productId, String wifiName, String wifiPwd);

    /**
     * 设置倒计时提醒
     *
     * @param device  设备
     * @param seconds 倒计时（秒）
     */
    boolean setVoice(IotDevice device, long seconds);

    boolean setVoice(String mac, String productId, long seconds);

    /**
     * 设置总用电量
     * @param device 设备
     * @param totalEle 总用电量（瓦时）
     */
    boolean setTotalEle(IotDevice device, BigDecimal totalEle);

    /**
     * 设置总用电量
     * @param deviceName 设备MAC
     * @param productId 产品ID
     * @param totalEle 总用电量（瓦时）
     */
    boolean setTotalEle(String deviceName, String productId, BigDecimal totalEle);

    /**
     * 获取历史数据
     */
    HistoryDeviceData getHistoryDataPoint(String deviceName, String productId);

    /**
     * 设置设备时长和电量
     * @param device 设备
     * @param seconds 时长（秒）
     * @param ele 电量（度）
     */
    int setTimeAndEle(IotDevice device, long seconds, BigDecimal ele);

    /**
     * 设置设备时长和电量
     * @param mac mac
     * @param productId 产品ID
     * @param seconds 秒
     * @param ele 电量（度）
     */
    int setTimeAndEle(String mac, String productId, long seconds, BigDecimal ele);

    /**
     * 设置电压系数
     */
    CommandResponse setVxs(DeviceVO device, BigDecimal vxs, String reason);

    /**
     * 上传数据
     */
    CommandResponse uploadData(IotDevice device, String reason, boolean recordLog);

}
